--[=[
Name: LibTrueRandom-1.0
Revision: 1
Author(s): ckknight (ckknight@gmail.com)
Description: A library that provides a way to get server-side random values.
Dependencies: LibStub
]=]

local MAJOR_VERSION = "LibTrueRandom-1.0"
local MINOR_VERSION = 2

local _G = _G
if not _G.LibStub then _G.error(MAJOR_VERSION .. " requires LibStub.") end
local LibTrueRandom = _G.LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not LibTrueRandom then
	return
end

local frame = LibTrueRandom.frame or _G.CreateFrame("Frame")
LibTrueRandom.frame = frame
frame:UnregisterAllEvents()
frame:RegisterEvent("CHAT_MSG_SYSTEM")

local minimums = LibTrueRandom.minimums or {}
LibTrueRandom.minimums = minimums
local maximums = LibTrueRandom.maximums or {}
LibTrueRandom.maximums = maximums
local callbacks = LibTrueRandom.callbacks or {}
LibTrueRandom.callbacks = callbacks

local parser
do
	local unpattern = '^' .. _G.RANDOM_ROLL_RESULT:gsub("([%(%)%.%*%+%-%[%]%?%^%$%%])", "%%%1") .. '$'
	if not unpattern:match("%%%%1%%%$[sd]") then
		unpattern = unpattern:gsub("%%%%[sd]", "(.*)")

		function parser(text)
			return text:match(unpattern)
		end
	else
		local indexes = {}
		unpattern = unpattern:gsub("%%%%([1234])%%%$[sd]", function(i)
			indexes[#indexes+1] = i+0
			return "(.*)"
		end)

		if indexes[1] == 1 and indexes[2] == 2 and indexes[3] == 3 and indexes[4] == 4 then
			function parser(text)
				return text:match(unpattern)
			end
		else
			function parser(text)
				local a, b, c, d = text:match(unpattern)
				if a == nil then
					return nil, nil, nil, nil
				end

				local data = { a, b, c, d }
				return data[indexes[1]], data[indexes[2]], data[indexes[3]], data[indexes[4]]
			end
		end
	end
end

local math_floor = _G.assert(_G.math.floor)
local error = _G.assert(_G.error)
local tonumber = _G.assert(_G.tonumber)
local table_remove = _G.assert(_G.table.remove)
local type = _G.assert(_G.type)
local tostring = _G.assert(_G.tostring)

local player_name = _G.UnitName("player")
frame:SetScript("OnEvent", function(self, event, text)
	local name, amount, minimum, maximum = parser(text)

	if not name or name ~= player_name then
		return
	end

	amount = tonumber(amount)
	minimum = tonumber(minimum)
	maximum = tonumber(maximum)

	if not amount or not minimum or not maximum or math_floor(amount) ~= amount or amount < minimum or amount > maximum then
		return
	end

	for i = 1, #minimums do
		if minimums[i] == minimum and maximums[i] == maximum then
			table_remove(minimums, i)
			table_remove(maximums, i)
			local callback = table_remove(callbacks, i)
			callback(amount)
			break
		end
	end
end)

local MINIMUM_RANDOM_VALUE = 1
local MAXIMUM_RANDOM_VALUE = 1000000
--- Asychronously send a random request to the server and call the provided callback when the result is returned.
-- @param minimum The minimum number. Must be at least 1 and must be an integer.
-- @param maximum The maximum number. Must be greater than the minimum, no greater than 1000000, and must be an integer.
-- @param callback A function to call when the result is known.
-- @usage LibTrueRandom.GetRandom(1, 10, function(value) print("Got random: " .. value) end)
function LibTrueRandom.GetRandom(minimum, maximum, callback)
	if type(minimum) ~= "number" or math_floor(minimum) ~= minimum or minimum < MINIMUM_RANDOM_VALUE then
		error(("Argument #1 (minimum) to `GetRandom' must be an integer and at least %s, got %s (%s)"):format(MINIMUM_RANDOM_VALUE, type(minimum), tostring(minimum)), 2)
	elseif type(maximum) ~= "number" or math_floor(maximum) ~= maximum or maximum <= minimum or maximum > MAXIMUM_RANDOM_VALUE then
		error(("Argument #2 (maximum) to `GetRandom' must be an integer, greater than the minimum (%s), and at most %s, got %s (%s)"):format(minimum, MAXIMUM_RANDOM_VALUE, type(maximum), tostring(maximum)), 2)
	elseif type(callback) ~= "function" then
		error(("Argument #3 (callback) to `GetRandom' must be a function, got %s (%s)"):format(type(callback), tostring(callback)), 2)
	end
	
	local i = #minimums + 1

	minimums[i] = minimum
	maximums[i] = maximum
	callbacks[i] = callback
	
	_G.RandomRoll(minimum, maximum)
end